==> cryptix-asn1-0.1.8.zip: src/cryptix/asn1/lang/SetOf.java <==
/* $Id: SetOf.java,v 1.2 2001/04/11 20:39:48 raif Exp $
 *
 * Copyright (C) 1997-2001 The Cryptix Foundation Limited. All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject to
 * the terms and conditions of the Cryptix General Licence. You should have
 * received a copy of the Cryptix General Licence along with this library; if
 * not, you can download a copy from http://www.cryptix.org/
 */
package cryptix.asn1.lang;

import cryptix.asn1.io.ASNReader;
import cryptix.asn1.io.ASNWriter;
import cryptix.asn1.io.ElementNotFoundException;

import org.apache.log4j.Category;

import java.io.EOFException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;

/**
 * The abstract implementation of the ASN.1 SET OF type.<p>
 *
 * The value of such type is a <tt>java.util.ArrayList</tt>. This data
 * structure is supposed to hold the concrete value sets of 0 or more instances
 * of the SET OF components. For example, if this is an instance of SET OF
 * TypeA, then the <tt>components</tt> of this SET OF contains <i>TypeA</i>,
 * and its value is a <tt>java.util.ArrayList</tt> of 0 or more values of
 * <i>TypeA</i>. The <tt>components</tt> field of this type is itself a
 * <tt>java.util.ArrayList</tt>.<p>
 *
 * A convenience method to return the value as a <tt>java.util.ArrayList</tt>
 * is supplied.<p>
 *
 * @version $Revision: 1.2 $
 * @author  Raif S. Naffah
 */
public abstract class SetOf extends CompoundType {

	// Constants and vars
	// .......................................................................

	static Category cat = Category.getInstance(SetOf.class.getName());

	// Constructor(s)
	// .......................................................................

	public SetOf(Tag tag) {
		super("", tag);
	}

	public SetOf(String name, Tag tag) {
		super(name, tag);
	}

	/**
	 * Constructs a new instance of a SET_OF type, given a designated Tag and a
	 * designated initial value.
	 *
	 * @param name the name of this instance.
	 * @param tag the designated Tag value.
	 * @param value the designated initial value.
	 * @exception ClassCastException if the designated value is not an ArrayList.
	 */
	public SetOf(String name, Tag tag, Object value) {
		super(name, tag);

		if (value != null)
			if (value instanceof ArrayList)
				setValue(value);
			else
				throw new ClassCastException();
	}

	// Redefinition of methods in superclass Type
	// .......................................................................

	/**
	 * Decodes a SET OF from an input stream.
	 *
	 * @param is the ASN.1 stream to read from.
	 * @exception IOException if an exception occurs during the operation.
	 */
	public void decode(ASNReader is) throws IOException {
		String cn = this.getClass().getName();
		cat.debug("==> "+cn+".decode()");

		this.reset();

		is.mark(Integer.MAX_VALUE);
		ASNReader local = is.decodeStructure(this);
		if (local == null) {
			cat.warn(cn+" not found. Backtracking...");
			is.reset();
			throw new ElementNotFoundException(cn);
		}

      // buggy
//		Iterator it;
//		IType t;
//		outer: while (true) {
//			ArrayList e = cloneComponents();
//			cat.info("Looking for a valid "+cn+" (SET OF) frame...");
//			inner: while (true)
//				try {
//					IType et = local.decodeAny("???");
//					cat.info("Decoded from stream: "+String.valueOf(et));
//					it = e.iterator();
//					while (it.hasNext()) {
//						t = (Type) it.next();
//						cat.info("Comparing it with "+String.valueOf(t));
//						cat.debug("t.isBlank() = "+t.isBlank());
//						cat.debug("t.getTag() = "+t.getTag());
//						cat.debug("et.getTag() = "+et.getTag());
//						if (t.isBlank() && et.getTag().equals(t.getTag())) {
//							t.setValue(et.getValue());
//							cat.info("Found a "+t.getClass().getName());
//							continue inner;
//						}
//					}
//					// it's not the end of file + we went thru all the components
//					// but found no match. if none of the elements have a value
//					// then it's time to quit with an exception, otherwise clone
//					// and continue.
//					// first make sure that, in this frame, all non-optional types
//					// have values.
//					for (it = e.iterator(); it.hasNext(); ) {
//						t = (Type) it.next();
//						if (t.isBlank() && !t.isOptional())
//							throw new ASNException("After completely parsing a "+cn
//							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
//							+") was found missing");
//					}
//
//					for (it = e.iterator(); it.hasNext(); )
//						if (!((Type) it.next()).isBlank()) {
//							cat.info("Eureka! Adding another frame to "+cn+"'s (SET OF) collection...");
//							this.iterator().add(e);
//							continue outer;
//						}
//
//					throw new ASNException("After completely parsing a "+cn
//						+" (SET OF) frame, an unknown type ("+et.getClass().getName()
//						+") was found");
//				} catch (EOFException x) {
//					for (it = e.iterator(); it.hasNext(); ) {
//						t = (Type) it.next();
//						if (t.isBlank() && !t.isOptional())
//							throw new ASNException("After completely parsing the last "+cn
//							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
//							+") was found missing");
//					}
//
//					for (it = e.iterator(); it.hasNext(); )
//						if (!((Type) it.next()).isBlank()) {
//							cat.info("Eureka! Adding last frame to "+cn+"'s (SET OF) collection...");
//							this.iterator().add(e);
//							break outer;
//						}
//
//					break outer;
//				}
//		}

		Iterator it;
		IType t;
		int found;
		ArrayList e;
		outer: while (true) {
			e = cloneComponents();
			cat.info("Looking for a valid "+cn+" (SET OF) frame...");
			inner: while (true) {
      		for (it = e.iterator(), found = 0; it.hasNext(); ) {
               t = (Type) it.next();
               if (t.isBlank()) {
                  try {
                     local.mark(Integer.MAX_VALUE);
         				cat.info("Looking for "+t.getClass().getName());
                     t.decode(local);
         				cat.info("Found "+t.getClass().getName());
         				found++;
         			} catch (EOFException x) {
         			   cat.info("Exhausted the input stream. Breaking...");
      					for (it = e.iterator(); it.hasNext(); ) {
      						t = (Type) it.next();
      						if (t.isBlank() && !t.isOptional())
      							throw new ASNException("After completely parsing the last "+cn
      							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
      							+") was found missing");
      					}

      					for (it = e.iterator(); it.hasNext(); )
      						if (!((Type) it.next()).isBlank()) {
      							cat.info("Eureka! Adding last frame to "+cn+"'s (SET OF) collection...");
      							this.iterator().add(e);
      							break outer;
      						}

      					break outer;
                  } catch (ElementNotFoundException x) {
         				cat.warn(String.valueOf(x)+". Backtracking...");
         				if (!t.isOptional())
                        local.reset();
                  }
               }
            }

            if (found == 0) {
         	   cat.info("Found no match. Breaking...");
					for (it = e.iterator(); it.hasNext(); ) {
						t = (Type) it.next();
						if (t.isBlank() && !t.isOptional())
							throw new ASNException("After completely parsing the last "+cn
							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
							+") was found missing");
					}

					for (it = e.iterator(); it.hasNext(); )
						if (!((Type) it.next()).isBlank()) {
							cat.info("Eureka! Adding last frame to "+cn+"'s (SET OF) collection...");
							this.iterator().add(e);
							break outer;
						}

               break outer;
            }
         }
      }

		cat.debug("<== "+cn+".decode()");
	}

   // java.lang.Object overloaded methods
   // .......................................................................

	/**
	 * Returns a string representation of this instance.
	 *
	 * @return a string representation of this instance.
	 */
	public String toString() {
		String cn = this.getClass().getName();
		String ls = System.getProperty("line.separator");
		StringBuffer sb = new StringBuffer(ls);
		sb.append("-- Begin ").append(cn).append(" (SET OF)").append(ls);
		int i = 1;
		for (Iterator it = this.iterator(); it.hasNext(); ) {
			Object t = it.next();
			sb.append("-- frame #").append(i++)
				.append(" (").append(t.getClass().getName()).append("): ")
				.append(String.valueOf(t)).append(ls);
		}

		sb.append("-- End ").append(cn).append(" (SET OF)");
		return sb.toString();
	}
}

==> cryptix-asn1-0.1.9.zip: src/cryptix/asn1/lang/SetOf.java <==
/* $Id: SetOf.java,v 1.5 2001/05/06 05:49:28 raif Exp $
 *
 * Copyright (C) 1997-2001 The Cryptix Foundation Limited. All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject to
 * the terms and conditions of the Cryptix General Licence. You should have
 * received a copy of the Cryptix General Licence along with this library; if
 * not, you can download a copy from http://www.cryptix.org/
 */
package cryptix.asn1.lang;

import cryptix.asn1.io.ASNReader;
import cryptix.asn1.io.ASNWriter;
import cryptix.asn1.io.ElementNotFoundException;

import org.apache.log4j.Category;

import java.io.EOFException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;

/**
 * The abstract implementation of the ASN.1 SET OF type.<p>
 *
 * The value of such type is a <tt>java.util.ArrayList</tt>. This data
 * structure is supposed to hold the concrete value sets of 0 or more instances
 * of the SET OF components. For example, if this is an instance of SET OF
 * TypeA, then the <tt>components</tt> of this SET OF contains <i>TypeA</i>,
 * and its value is a <tt>java.util.ArrayList</tt> of 0 or more values of
 * <i>TypeA</i>. The <tt>components</tt> field of this type is itself a
 * <tt>java.util.ArrayList</tt>.<p>
 *
 * A convenience method to return the value as a <tt>java.util.ArrayList</tt>
 * is supplied.<p>
 *
 * @version $Revision: 1.5 $
 * @author  Raif S. Naffah
 */
public abstract class SetOf extends CompoundType {

	// Constants and vars
	// .......................................................................

	static Category cat = Category.getInstance(SetOf.class.getName());

	// Constructor(s)
	// .......................................................................

	public SetOf(Tag tag) {
		super("", tag);
	}

	public SetOf(String name, Tag tag) {
		super(name, tag);
	}

	/**
	 * Constructs a new instance of a SET_OF type, given a designated Tag and a
	 * designated initial value.
	 *
	 * @param name the name of this instance.
	 * @param tag the designated Tag value.
	 * @param value the designated initial value.
	 * @exception ClassCastException if the designated value is not an ArrayList.
	 */
	public SetOf(String name, Tag tag, Object value) {
		super(name, tag);

		if (value != null)
			if (value instanceof ArrayList)
				setValue(value);
			else
				throw new ClassCastException();
	}

	// Redefinition of methods in superclass Type
	// .......................................................................

	/**
	 * Decodes a SET OF from an input stream.
	 *
	 * @param is the ASN.1 stream to read from.
	 * @exception IOException if an exception occurs during the operation.
	 */
	public void decode(ASNReader is) throws IOException {
		String cn = this.getClass().getName();
		cat.debug("==> "+cn+".decode()");

		this.reset();

		is.mark(Integer.MAX_VALUE);
		ASNReader local = is.decodeStructure(this);
		if (local == null) {
			cat.warn(cn+" not found. Backtracking...");
			is.reset();
			throw new ElementNotFoundException(cn);
		}

      // buggy
//		Iterator it;
//		IType t;
//		outer: while (true) {
//			ArrayList e = cloneComponents();
//			cat.info("Looking for a valid "+cn+" (SET OF) frame...");
//			inner: while (true)
//				try {
//					IType et = local.decodeAny("???");
//					cat.info("Decoded from stream: "+String.valueOf(et));
//					it = e.iterator();
//					while (it.hasNext()) {
//						t = (Type) it.next();
//						cat.info("Comparing it with "+String.valueOf(t));
//						cat.debug("t.isBlank() = "+t.isBlank());
//						cat.debug("t.getTag() = "+t.getTag());
//						cat.debug("et.getTag() = "+et.getTag());
//						if (t.isBlank() && et.getTag().equals(t.getTag())) {
//							t.setValue(et.getValue());
//							cat.info("Found a "+t.getClass().getName());
//							continue inner;
//						}
//					}
//					// it's not the end of file + we went thru all the components
//					// but found no match. if none of the elements have a value
//					// then it's time to quit with an exception, otherwise clone
//					// and continue.
//					// first make sure that, in this frame, all non-optional types
//					// have values.
//					for (it = e.iterator(); it.hasNext(); ) {
//						t = (Type) it.next();
//						if (t.isBlank() && !t.isOptional())
//							throw new ASNException("After completely parsing a "+cn
//							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
//							+") was found missing");
//					}
//
//					for (it = e.iterator(); it.hasNext(); )
//						if (!((Type) it.next()).isBlank()) {
//							cat.info("Eureka! Adding another frame to "+cn+"'s (SET OF) collection...");
//							this.iterator().add(e);
//							continue outer;
//						}
//
//					throw new ASNException("After completely parsing a "+cn
//						+" (SET OF) frame, an unknown type ("+et.getClass().getName()
//						+") was found");
//				} catch (EOFException x) {
//					for (it = e.iterator(); it.hasNext(); ) {
//						t = (Type) it.next();
//						if (t.isBlank() && !t.isOptional())
//							throw new ASNException("After completely parsing the last "+cn
//							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
//							+") was found missing");
//					}
//
//					for (it = e.iterator(); it.hasNext(); )
//						if (!((Type) it.next()).isBlank()) {
//							cat.info("Eureka! Adding last frame to "+cn+"'s (SET OF) collection...");
//							this.iterator().add(e);
//							break outer;
//						}
//
//					break outer;
//				}
//		}

		Iterator it;
		IType t;
		int found;
		ArrayList e;
		outer: while (true) {
			e = cloneComponents();
			cat.info("Looking for a valid "+cn+" (SET OF) frame...");
			inner: while (true) {
      		for (it = e.iterator(), found = 0; it.hasNext(); ) {
               t = (Type) it.next();
               if (t.isBlank()) {
                  try {
                     local.mark(Integer.MAX_VALUE);
         				cat.info("Looking for "+t.getClass().getName());
                     t.decode(local);
         				cat.info("Found "+t.getClass().getName());
         				found++;
         			} catch (EOFException x) {
         			   cat.info("Exhausted the input stream. Breaking...");
      					for (it = e.iterator(); it.hasNext(); ) {
      						t = (Type) it.next();
      						if (t.isBlank() && !t.isOptional())
      							throw new ASNException("After completely parsing the last "+cn
      							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
      							+") was found missing");
      					}

      					for (it = e.iterator(); it.hasNext(); )
      						if (!((Type) it.next()).isBlank()) {
      							cat.info("Eureka! Adding last frame to "+cn+"'s (SET OF) collection...");
      							this.iterator().add(e);
      							break outer;
      						}

      					break outer;
                  } catch (ElementNotFoundException x) {
         				cat.warn(String.valueOf(x)+". Backtracking...");
         				if (!t.isOptional())
                        local.reset();
                  }
               }
            }

            if (found == 0) {
         	   cat.info("Found no match. Breaking...");
					for (it = e.iterator(); it.hasNext(); ) {
						t = (Type) it.next();
						if (t.isBlank() && !t.isOptional())
							throw new ASNException("After completely parsing the last "+cn
							+" (SET OF) frame, a non optional type ("+t.getClass().getName()
							+") was found missing");
					}

					for (it = e.iterator(); it.hasNext(); )
						if (!((Type) it.next()).isBlank()) {
							cat.info("Eureka! Adding last frame to "+cn+"'s (SET OF) collection...");
							this.iterator().add(e);
							break outer;
						}

               break outer;
            }
         }
      }

		cat.debug("<== "+cn+".decode()");
	}

	/**
	 * Sets the current value of this instance to the designated one. For a
	 * compound type, the value is expected to always be an ArrayList or an
	 * ANY with an ArrayList object as its value.
	 *
	 * @param value the new value to assign to this instance.
	 */
	public void setValue(Object value) {
		String cn = this.getClass().getName();
		cat.debug("==> "+cn+".setValue()");
		cat.debug("value="+String.valueOf(value));
      cat.debug("value is a "+String.valueOf(value.getClass().getName()));
      if (value instanceof IType)
         cat.debug("Tag="+String.valueOf( ((IType) value).getTag() ));

      this.reset();

      ArrayList e;
      Iterator frame;
      IType td;

      if (value instanceof IType) {
         value = ((IType) value).getValue();
   		cat.debug("value now="+String.valueOf(value));
         cat.debug("value is now a "+String.valueOf(value.getClass().getName()));
         if (value instanceof IType)
            cat.debug("Tag="+String.valueOf( ((IType) value).getTag() ));
      }

      if (value instanceof ArrayList) {
         cat.warn("Assume a SEQUENCE OF one component...");
         frame = ((ArrayList) value).iterator();
         for (int i = 1; frame.hasNext(); i++) {
   		   cat.info("Processing frame #"+String.valueOf(i)+" of "+cn);
      		td = (IType) cloneComponents().iterator().next();
      		cat.info("Assigning value to "+td.getClass().getName());
            td.setValue(frame.next());
            this.iterator().add(td);
         }
      } else {
         RuntimeException x = new RuntimeException();
         x.printStackTrace(System.err);
         throw x;
      }

////      if (value instanceof IType)
////         value = ((IType) value).getValue();
//
//		Iterator src;
//		if (value instanceof IIterativeType)
//		   src = ((IIterativeType) value).iterator();
//		else if (value instanceof ArrayList)
//		   src = ((ArrayList) value).iterator();
//		else
//		   src = ((ArrayList) ((Any) value).getValue()).iterator();
//
////      ArrayList e;
////      IType td;
////      Object ts;
////      for (int i = 1 ; src.hasNext(); i++) {
////   		cat.info("Processing frame #"+String.valueOf(i)+" of "+cn);
////   		e = cloneComponents();
////         ts = src.next();
////   		for (Iterator it = e.iterator(); it.hasNext(); ) {
////   		   td = (Type) it.next();
////   			cat.info("Assigning value from "+ts.getClass().getName()+" to "+td.getClass().getName());
////   			td.setValue(ts);
////         }
////
////		   this.iterator().add(e);
////   		cat.info("Assigned new frame to "+cn);
////      }
//
//      IType ts;
//      Iterator dst;
//      Object obj;
//      for (int i = 1 ; src.hasNext(); i++) {
//   		cat.info("Processing frame #"+String.valueOf(i)+" of "+cn);
//   		e = cloneComponents();
//   		dst = e.iterator();
//   		obj = src.next();
//   		if (obj instanceof ArrayList) {
//            frame = ((ArrayList) obj).iterator();
//   		   while (frame.hasNext() && dst.hasNext()) {
//      			ts = (IType) frame.next();
//      			td = (IType) dst.next();
//      			cat.info("Assigning value from "+ts.getClass().getName()+" to "+td.getClass().getName());
//      			td.setValue(ts.getValue());
//            }
//
//		      this.iterator().add(e);
//         } else {
//      		td = (IType) dst.next();
//            cat.warn("Assume a SET OF one component...");
//      		cat.info("Assigning value to "+td.getClass().getName());
////            td.setValue(((IType) obj).getValue());
//            td.setValue(obj);
//
//            this.iterator().add(td);
//         }
//
////		   this.iterator().add(e);
//   		cat.info("Assigned new frame to "+cn);
//      }

		cat.debug("<== "+cn+".setValue()");
	}

   // java.lang.Object overloaded methods
   // .......................................................................

	/**
	 * Returns a string representation of this instance.
	 *
	 * @return a string representation of this instance.
	 */
	public String toString() {
		String cn = this.getClass().getName();
		String ls = System.getProperty("line.separator");
		StringBuffer sb = new StringBuffer(ls);
		sb.append("-- Begin ").append(cn).append(" (SET OF)").append(ls);
		int i = 1;
		for (Iterator it = this.iterator(); it.hasNext(); ) {
			Object t = it.next();
			sb.append("-- frame #").append(i++)
				.append(" (").append(t.getClass().getName()).append("): ")
				.append(String.valueOf(t)).append(ls);
		}

		sb.append("-- End ").append(cn).append(" (SET OF)");
		return sb.toString();
	}
}
